package com.cloveretl.customcomponent;

import org.jetel.data.DataRecord;
import org.jetel.data.DataRecordFactory;
import org.jetel.exception.AttributeNotFoundException;
import org.jetel.exception.ConfigurationStatus;
import org.jetel.exception.XMLConfigurationException;
import org.jetel.graph.InputPort;
import org.jetel.graph.Node;
import org.jetel.graph.Result;
import org.jetel.graph.TransformationGraph;
import org.jetel.metadata.DataFieldType;
import org.jetel.util.SynchronizeUtils;
import org.jetel.util.property.ComponentXMLAttributes;
import org.w3c.dom.Element;

/**
 * A sample custom component
 * 
 * Copies records from input to output, adding a "Hello" message to every string field
 * 
 */
public class HelloWorldComponent extends Node {

	/**
	 * Component internal type ID, see getType()
	 */
	public final static String COMPONENT_TYPE = "HELLO_WORLD";

	/**
	 * The message to say.
	 * Can be overridden by setting "Message to say" attribute of the component.
	 * 
	 * @see customcomponent.xml and "defaultHint" attribute - the default value here MUST be the same as in defaultHint  
	 */
	private String message = "Hello";

	/**
	 * Name of the "Message to say" attribute in the XML
	 * 
	 * @see fromXML() and customcomponent.xml
	 */
	public final static String XML_MESSAGE_ATTRIBUTE = "message";

	
	/**
	 * Standard constructor
	 * 
	 * @param id
	 * 		  Component ID in the graph
	 */
	public HelloWorldComponent(String id) {
		super(id);
	}

	/**
	 * Our constructor with custom message
	 * 
	 * @param id
	 *        Component ID in the graph
	 *        
	 * @param message
	 *        Custom message to say
	 */
	public HelloWorldComponent(String id, String message) {
		super(id);
		
		if (message != null && message.trim().length() > 0) {
			this.message = message;
		}
	}

	
	@Override
	public Result execute() throws Exception {

		/**
		 *  Initialize input port and a bucket input record that will be field with records coming to input port
		 */
		InputPort inPort = (InputPort) getInputPort(0); // ports are numbered from 0
		DataRecord inRecord = DataRecordFactory.newRecord(inPort.getMetadata()); // always use DataRecordFactory to create record instances
		int numFields = inRecord.getNumFields();
		inRecord.init(); // initialize records only once
		inRecord.reset(); // just to make sure all field values are reset
		
		/**
		 * Initialize output and "bucket" output records that will be filled as "copy-by-name" from the input with additional message
		 */
		int numOutputPorts=getOutPorts().size();
		DataRecord[] outRecord = new DataRecord[numOutputPorts]; 
		for (int i = 0; i < numOutputPorts; i++) {
			outRecord[i] = DataRecordFactory.newRecord(getOutputPort(i).getMetadata());
			outRecord[i].init();
			outRecord[i].reset();
		}
		
		/**
		 * MAIN LOOP
		 */
		while (inPort.readRecord(inRecord) != null && runIt) { // always check internal "runIt" state 
			
			/**
			 * Add "message" to every field in the input which has "string" data type 
			 */
			for(int fieldNo = 0; fieldNo < numFields; fieldNo++) {
				if (inRecord.getField(fieldNo).getMetadata().getDataType().equals(DataFieldType.STRING)) {
					inRecord.getField(fieldNo).setValue(message + " " + String.valueOf(inRecord.getField(fieldNo).getValue()));
				}
			}
			
			/**
			 * Send output to all out ports
			 * For each output port, copy the modified input record (by name) and then send it out
			 */
			for (int i=0; i < numOutputPorts; i++) {
			    outRecord[i].reset(); // to make sure we have clean record before copying
			    outRecord[i].copyFieldsByName(inRecord);
			    writeRecord(i, outRecord[i]); // sends out record to the output edge
			}
			
			/**
			 * Mandatory yield after every record
			 */
			SynchronizeUtils.cloverYield();
		}
		
        return runIt ? Result.FINISHED_OK : Result.ABORTED;
	}

	
	/**
	 * Factory method that creates the component from transformation graph source XML
	 */
	public static Node fromXML(TransformationGraph graph, Element xmlElement) throws XMLConfigurationException, AttributeNotFoundException {
		
		ComponentXMLAttributes xattribs = new ComponentXMLAttributes(xmlElement, graph);

		String id = xattribs.getString(XML_ID_ATTRIBUTE);
		String message = xattribs.exists(XML_MESSAGE_ATTRIBUTE) ? xattribs.getString(XML_MESSAGE_ATTRIBUTE) : null;
	
		return new HelloWorldComponent(id, message);
	}


	/**
	 * checkConfig checks configuration of the component for Designer and Engine.
	 * Here, you would check connected input and output ports, required metadata etc.
	 */
    @Override
    public ConfigurationStatus checkConfig(ConfigurationStatus status) {
		super.checkConfig(status);
	 
		/**
		 * Check that we have one input port and at least one output
		 */
		if(!checkInputPorts(status, 1, 1) || !checkOutputPorts(status, 1, Integer.MAX_VALUE, false)) {
			return status;
		}

        return status;
    }
	
	@Override
	public String getType(){
		return COMPONENT_TYPE;
	}
	
}